## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Additional Functions added by Ben McJunkin for SEVOMOD 3.0
##

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup
import pickle
import DataStorage
import CvCameraControls

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
data = DataStorage.DataStorage()
PyInfo = PyHelpers.PyInfo
dice = gc.getGame().getMapRand()

class BTM_TechSpread:
	def onBeginPlayerTurn(self, argsList):
		iGameTurn, iPlayer = argsList

		# iTechPercentModifier indicates what percent of tech knowledge may be passed to trading partners
		iTechPercentModifier = 0.003 

		# Modify iTechPercentModifier to match game speed (originally tested for marathon, thus the reason for 1200 factor
		iTechPercentModifier = iTechPercentModifier * 1200 / CyGame().getMaxTurns()
 
		# bFlagTechLeakShowReceiverMessages - if TRUE, show receiving player message that tech points were received
		bFlagTechLeakShowReceiverMessages = true

		# bFlagTechLeakShowSenderMessages - if TRUE, show sending player message that tech points were sent
		bFlagTechLeakShowSenderMessages = false

		pPlayer = gc.getPlayer(iPlayer)

		# Loop through all techs
		for iTechCursor in range(gc.getNumTechInfos()):
			# If pPlayer has tech iTechCursor
			if (not pPlayer.isNone() and not pPlayer.isBarbarian() and gc.getTeam(pPlayer.getTeam()).isHasTech(iTechCursor)):
				# Loop through all players
				for iTargetPlayer in range(gc.getMAX_PLAYERS()):
					pTargetPlayer = gc.getPlayer(iTargetPlayer)
					# If pTargetPlayer is a valid target to pass techs and does not yet have iTechCursor
					if (not pTargetPlayer.isNone() and pTargetPlayer.isAlive() and not pTargetPlayer.isBarbarian() and not gc.getTeam(pTargetPlayer.getTeam()).isHasTech(iTechCursor) and pTargetPlayer.canResearch(iTechCursor, false)):
						pTeam = gc.getTeam(pPlayer.getTeam())

						# Initialize amount to add variable, this will keep track of total points to add to this tech
						iAmtToAdd = 0
						# Cycle through all of pPlayer's cities
						for iCityCursor in range(pPlayer.getNumCities()):
							pPlayerCity = pPlayer.getCity(iCityCursor)
							# If pPlayerCity is valid
							if (not pPlayerCity.isNone()):
								pTargetTeam = gc.getTeam(pTargetPlayer.getTeam())
								# Loop through all of pPlayerCity's trade routes
								for iTargetCityCursor in range(pPlayerCity.getTradeRoutes()):
									pTargetCity = pPlayerCity.getTradeCity(iTargetCityCursor)

									# If pTargetCity is valid, and pTargetCity is owned by pTargetPlayer
									if (not pTargetCity.isNone() and pTargetCity.getOwner() == pTargetPlayer.getID()):
										# Add percentage into amount to add
										iAmtToAdd = iAmtToAdd + int(pTargetTeam.getResearchCost(iTechCursor) * iTechPercentModifier)

										# Can never auto-grant, so set to 1 less if meets or exceeds amount needed
										if (pTargetTeam.getResearchProgress(iTechCursor) + iAmtToAdd >= pTargetTeam.getResearchCost(iTechCursor)):
											iAmtToAdd = pTargetTeam.getResearchCost(iTechCursor) - pTargetTeam.getResearchProgress(iTechCursor) - 1
										# end if									
									# end if
								# end for
							# end if
						# end for

						# If there are points to be added, do so
						if (iAmtToAdd > 0):
							pTargetTeam.changeResearchProgress(iTechCursor, iAmtToAdd, pTargetPlayer.getID())

							# If message to tech receiving player is turned on, send message
							if (bFlagTechLeakShowReceiverMessages):
								CyInterface().addMessage(pTargetPlayer.getID(), false, 2, "You have gained %d points towards tech %s from %s!" %(iAmtToAdd, gc.getTechInfo(iTechCursor).getDescription(), pPlayer.getName()), "", 0, "Art/Interface/Buttons/Buildings/Palace.dds", CyGame().getReplayMessageColor(0), pTargetPlayer.getCapitalCity().getX(), pTargetPlayer.getCapitalCity().getY(), True, True)
							# end if

							# If message to tech sending player is turned on, send message
							if (bFlagTechLeakShowSenderMessages):
								CyInterface().addMessage(iPlayer, false, 2, "%s has gained %d points towards tech %s from trade with you!" %(pTargetPlayer.getName(), iAmtToAdd, gc.getTechInfo(iTechCursor).getDescription()), "", 0, "Art/Interface/Buttons/Buildings/Palace.dds", CyGame().getReplayMessageColor(0), pPlayer.getCapitalCity().getX(), pPlayer.getCapitalCity().getY(), True, True)
							# end if
						# end if
					# end if
				# end for
			# end if
		# end for


